<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services;

use App\Jobs\BhSorderRefundJob;
use App\Repositories\Contracts\BhPordersRepository;
use App\Repositories\Contracts\BhSoitemsRepository;
use App\Repositories\Contracts\BhSorderRefundRepository;
use App\Repositories\Contracts\BhSordersRepository;
use App\Repositories\Criteria\BhPordersCriteria;
use App\Repositories\Criteria\BhSorderRefundCriteria;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\AdminUsers;
use App\Repositories\Models\BhPorders;
use App\Support\Traits\Helpers;
use http\Exception\BadQueryStringException;
use Illuminate\Support\Facades\DB;


class BhOrdersService
{

    use Helpers;

    protected $bhPordersRepository = null;

    public function __construct(BhPordersRepository $bhPordersRepository)
    {
       $this->bhPordersRepository = $bhPordersRepository;
    }

    public function searchPordersToPage($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $this->bhPordersRepository->pushCriteria(new BhPordersCriteria($request));
        $pOrdersList = $this->bhPordersRepository->paginate($params['page_size']);

        $returnRecord = [];
        if($pOrdersList){
            $record = $this->procOrderResponseData($pOrdersList->items());
        }
        // 获取当前角色的所有权限
        $returnRecord['data'] = $this->formatKeysfromArray($record);
        $returnRecord['current_page'] = $pOrdersList->currentPage();
        $returnRecord['total'] = $pOrdersList->total();

        return $returnRecord;
    }

    /**
     * 编辑订单信息
     * @param $request
     * @return bool|int
     */
    public function editBhPordersInfo($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $order = BhPorders::query()->where('po_no',$params['po_no'])->get();
        if(!$order){
            throw new \LogicException('订单号不存在!',500);
        }

        $updateOrderArr = [];
        if(isset($params['po_sup_order']) && !empty($params['po_sup_order'])){
            $updateOrderArr['po_sup_order'] = $params['po_sup_order'];
        }

        $updateOrderArr['po_if_direct'] = 0;
        if(isset($params['po_if_direct']) && !empty($params['po_if_direct'])){
            if($params['po_if_direct'] == true){
                $updateOrderArr['po_if_direct'] = 1;
            }
        }

        $res = $order->first()->update($updateOrderArr);

        return $this->procOrderResponseData($order);
    }

    /**
     * 获取订单详情
     * @param $request
     */
    public function getBhSorderDetail($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');
        if(!isset($params['order_no']) || empty($params['order_no'])){
            throw new BadQueryStringException('订单编号错误',500);
        }

        $sOrder = app(BhSordersRepository::class)->getSorderFromOrderNo($params['order_no']);
        if(!$sOrder){
            throw new \LogicException('该订单编号不存在!',500);
        }

        //查询订单的详情
        $soItems = app(BhSoitemsRepository::class)->getSorderItemsDetailFromOrderId($sOrder->so_id);
        $soItems = $soItems->toArray();
        $returnItem = [];
        if($soItems){
            foreach($soItems as $item){
                $item['so_no'] = $sOrder->so_no;
                if($item['si_if_cancel'] == 0){
                    $returnItem[] = $item;
                }
            }
        }

       return $returnItem;
    }

    public function addSordersRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $sOrder = app(BhSordersRepository::class)->getSorderFromOrderNo($params['order_no']);
        if(!$sOrder){
            throw new \LogicException('该订单编号不存在!',500);
        }

        if($params['transfer_pre'] == true){
            $params['transfer_pre'] = 1;
        }else{
            $params['transfer_pre'] = 0;
        }

        if($params['cancel_porder'] == true){
            $params['cancel_porder'] = 1;
        }else{
            $params['cancel_porder'] = 0;
        }

        return app(BhSorderRefundRepository::class)->addRefundTask($params);
    }

    /**
     * 获取订单退货任务
     * @param $request
     * @return array
     */
    public function getSordersRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        if(!$params['page_size'] || $params['page_size'] == 0){
            $params['page_size'] = 10;
        }

        $request->attributes->add(['type' => 'sorder_refund']);
        $service = app(BhSorderRefundRepository::class);
        $service->pushCriteria(new BhSorderRefundCriteria($request));
        $tasks = $service->paginate($params['page_size']);
        $returnRecord = [];
        if($tasks){
            $allItems = $tasks->items();

            $allItems = collect($allItems)->map(function($item){
                $item = $item->toArray();

                $contentArr= [];
                $execArr = json_decode($item['exec_content'],true);
                foreach($execArr as $key => $content){
                    switch($key){
                        case 'order_no' : $contentArr[] = '【订单号】：'.$content; break;
                        case 'refund_nums' : $contentArr[] = '【退货数量】：'.$content; break;
                        case 'handle_fee' : $contentArr[] = '【手续费费用】：'.$content; break;
                        case 'item_id' : $contentArr[] = '【订单购买项id】：'.$content; break;
                        case 'transfer_pre' :
                            $content = $content == '1' ? '是' : '否';
                            $contentArr[] = '【是否转预存】：'.$content; break;
                        case 'cancel_porder' :
                            $content = $content == '1' ? '是' : '否';
                            $contentArr[] = '【是否取消采购订单】：'.$content; break;
                    }
                }

                $user = AdminUsers::find($item['create_by']);
                $item['create_by'] = $user->username;

                $item['content_string'] = implode(' . ',$contentArr);
                $item = array_merge($item,$execArr);

                return $item;
            });

            // 获取当前角色的所有权限
            $returnRecord['data'] = $this->formatKeysfromArray($allItems);
            $returnRecord['current_page'] = $tasks->currentPage();
            $returnRecord['total'] = $tasks->total();
        }

        return $returnRecord;
    }

    /**
     * 手动执行脚本
     * @param $request
     */
    public function execSordersRefundTask($request)
    {
        $params = $this->formatKeysfromArray($request->all(),'toUnderScore');

        $orderRefund = app(BhSorderRefundRepository::class)->find($params['task_id']);

        $refundJob = (new BhSorderRefundJob($orderRefund->toArray()))->delay(100)->onQueue('slow');
        app('Illuminate\Contracts\Bus\Dispatcher')->dispatch($refundJob);
    }

    /**
     * 处理退货逻辑
     * @param $refundInfo
     */
    public function sOrderRefund($refundInfo)
    {
        $sOrder = app(BhSordersRepository::class)->getSorderItemsFromOrderNo($refundInfo->order_no);
        if(!$sOrder){
            throw new \LogicException('该订单号不存在!',500);
        }

        try{

            $dbConnect = DB::connection('bh_mysql');
            $dbConnect->beginTransaction();

            // 处理订单退货项
            foreach($sOrder->toArray() as $orderitem){
                if($refundInfo->item_id == $orderitem['si_id']){

                    // 先处理源数据
                    $currentPrice = floatval($orderitem['si_discount']);
                    $currentAmount = floatval($orderitem['si_amount']);
                    // 减去退货数量，剩余的更新数量
                    $num = intval($orderitem['si_num']) - intval($refundInfo->refund_nums);

                    //查询是否已经开票，如果开票不能进行退货
                    $dpdetail = $dbConnect->table('dpdetail')
                        ->where('si_id',$orderitem['si_id'])
                        ->get();
                    $dpdetail = $dpdetail->toArray();
                    if(count($dpdetail) > 0){
                        foreach($dpdetail as $detail){
                            if(floatval($detail->dpd_invoiced) > 0) {
                                throw new \logicexception('以开票的数据不能退货!',500);
                            }
                        }
                    }
                    //修改item数据项
                    $service = app(BhSoitemsRepository::class);
                    $soItem = $service->findWhere(['si_id' => $orderitem['si_id'],'si_if_cancel' => 0]);

                    $soItem = current($soItem->toArray());

                    $updateArr = [];
                    $updateArr['si_num'] = $num == 0 ? 1 :$num; // 如果数量只有1，且退货数量相等
                    $updateArr['si_assigned_num'] = $updateArr['si_num'];
                    $updateArr['si_amount'] = bcmul(
                        $orderitem['si_discount'] ,
                        $updateArr['si_num'],
                        2
                    );
                    if(isset($refundInfo->handle_fee) && $refundInfo->handle_fee > 0){
                        $updateArr['si_amount'] = bcadd(
                            $updateArr['si_amount'] , $refundInfo->handle_fee,2
                        );
                    }
                    if($num > 1){
                        $updateArr['si_discount'] = bcdiv($updateArr['si_amount'] , $num, 2);
                    }else{
                        $updateArr['si_discount'] = $updateArr['si_amount'];
                    }
                    $service->update($updateArr,$soItem['si_id']);

                    //生成新记录
                    $createArr = $soItem;
                    unset($createArr['si_id']);

                    if(isset($refundInfo->handle_fee) && floatval($refundInfo->handle_fee) > 0){
                        if($num == 0){
                           $createArr['si_amount'] = $refundInfo->handle_fee;
                        }else{
                            $createArr['si_amount'] = bcmul($refundInfo->refund_nums, $soItem['si_discount'], 2);
                        }
                        $createArr['si_discount'] = bcdiv($createArr['si_amount'], $createArr['si_num'], 2);
                    } // 没有手续费的情况下
                    else{
                        $createArr['si_discount'] = $soItem['si_discount'];
                        $createArr['si_amount'] = bcmul($soItem['si_discount'], $refundInfo->refund_nums, 2);
                    }
                    $createArr['si_num'] = $refundInfo->refund_nums;
                    $createArr['si_assigned_num'] = $createArr['si_num'];

                    if($createArr['si_amount'] < 0){
                        throw new \LogicException('手续费异常,处理失败!',500);
                    }

                    $newItem = $service->create($createArr);

                    //修改配货单记录
                    if($dpdetail){
                        $maxRecord = $minRecord = [];
                        $needUpdate = null;
                        $totalNums = $totalAmount =  0;
                        //针对多个dpdetail数据，进行数量判断
                        //默认都是没有开票过的
                        foreach($dpdetail as $detail){
                            $detailArr = [];
                            $totalNums += $detail->dpd_num;
                            $totalAmount += $detail->dpd_amount;
                            foreach($detail as $key => $val){
                               $detailArr[$key] = $val;
                            }

                            if($detail->dpd_num == $refundInfo->refund_nums){
                                $needUpdate = $detailArr;
                                break;
                            }
                            // 1 数量5， 2条记录 3/2  , 退货4瓶
                            // 2 数量5， 2条记录 4/1  , 退货3瓶
                            // 3 数量6   3条记录 4/1/1, 退货3瓶
                            // 4 数量8   3条记录 4/2/2, 退货4瓶
                            // 5 数量8   3条记录 4/3/1, 退货4瓶
                            // 6 数量11  3条记录 6/4/1, 退货5瓶
                            if($detail->dpd_num > $refundInfo->refund_nums){
                                $nums = $detail->dpd_num;
                                $maxRecord[$nums][$detail->dpd_id] = $detailArr;
                            }elseif($refundInfo->refund_nums > $detail->dpd_num){
                                $nums = $refundInfo->refund_nums - $detail->dpd_num;
                                $minRecord[$nums][$detail->dpd_id] = $detailArr;
                            }
                        }

                        $cancelRecord = [];
                        // 如果退货数量跟此明细数量相同
                        if(!is_null($needUpdate)){
                            $dpUpdateArr = [];
                            $dpUpdateArr['si_id'] = $newItem->toArray()['si_id'];
                            // 如果有手续费，对价格进行更新
                            if($refundInfo->handle_fee > 0){
                                // 如果有手续费，将价格进行修改
                                foreach($dpdetail as $detail){
                                   if($detail->dpd_id != $needUpdate['dpd_id']){
                                      $otherUpdate = [];
                                      $otherUpdate['dpd_amount'] = bcmul($updateArr['si_discount'], $needUpdate['dpd_num'], 2);
                                       $dbConnect->table('dpdetail')
                                           ->where('dpd_id',$dpdetail->dpd_id)
                                           ->update($otherUpdate);
                                   }
                                }
                                //将取消的记录价格进行更新
                                $dpUpdateArr['dpd_amount'] = bcmul($createArr['si_discount'], $needUpdate['dpd_num'], 2);
                                $dbConnect->table('dpdetail')
                                    ->where('dpd_id',$needUpdate['dpd_id'])
                                    ->update($dpUpdateArr);
                            } // 如果没有手续费，不对价格进行更新
                            else{
                                $dbConnect->table('dpdetail')
                                    ->where('dpd_id',$needUpdate['dpd_id'])
                                    ->update($dpUpdateArr);
                            }
                        }else{

                            asort($maxRecord);
                            arsort($minRecord); // 降序排序

                            $num = $refundInfo->refund_nums;
                            // minrecord有值，只有退货数量大于此条记录的数量时
                            $cancelRecord = array_merge($cancelRecord,$this->handleDpdetailInfo($dbConnect,$num,$minRecord,$newItem->toArray()['si_id'],$createArr,$updateArr,$refundInfo,'asort'));
                            $cancelRecord = array_merge($cancelRecord,$this->handleDpdetailInfo($dbConnect,$num,$maxRecord,$newItem->toArray()['si_id'],$createArr,$updateArr,$refundInfo,'arsort'));
                        }

                        // 更新除了取消的记录以外的记录的价格
                        if(count($cancelRecord) > 0){
                            $dpdetailInfo = $dbConnect->table('dpdetail')
                                ->where('si_id',$orderitem['si_id'])
                                ->whereNotIn('dpd_id',$cancelRecord)->get();
                            $dpdetailInfo = $dpdetailInfo->toArray();
                            if(count($dpdetailInfo) > 0){
                                foreach($dpdetailInfo as $info){
                                    $otherUpdate = [];
                                    $otherUpdate['dpd_amount'] = bcmul($updateArr['si_discount'], $info->dpd_num, 2);
                                    $dbConnect->table('dpdetail')
                                        ->where('dpd_id',$info->dpd_id)
                                        ->update($otherUpdate);
                                }
                            }
                        }
                    }

                    //新增配货单数据
                    /*foreach($dpdetail as $key => $value){
                        $dpCreateArr[$key] = $value;
                    }
                    unset($dpCreateArr['dpd_id']);

                    $dpCreateArr['dpd_num'] = $refundInfo->refund_nums;
                    $dpCreateArr['dpd_amount'] = $createArr['si_amount'];

                    $dpId = $dbConnect->table('dpdetail')->insertGetId($dpCreateArr);*/

                    // 修改采购订单的详情数据
                    $soItemArr = $soItem;
                    $poItems = $dbConnect->table('poitems')
                        ->where('p_id', $soItemArr['p_id'])
                        ->where('pi_id', $soItemArr['pi_id'])
                        ->where('pi_status','<>',3)
                        ->get();

                    if($poItems){
                        foreach($poItems as $item){
                            // 如果不是取消状态的
                            if($item->pi_status != 3){
                                // 可能存在对应多个po的情况
                                if($item->pi_num < $refundInfo->refund_nums){
                                    throw new \LogicException('采购订单数量不足,无法退货!',500);
                                }

                                // 如果退货数量小于采购订单数量
                                if($item->pi_num >= $refundInfo->refund_nums){
                                    // 修改当前的数量和金额
                                    $poItemUpdateArr = [];
                                    $poNums = bcsub($item->pi_num, $refundInfo->refund_nums);
                                    $poItemUpdateArr['pi_num'] = $poNums;
                                    $poItemUpdateArr['pi_amount'] = bcmul($item->pi_price, $poNums, 2);
                                    $poItemUpdateArr['pi_rnum'] = $poItemUpdateArr['pi_num'];
                                    if($poItemUpdateArr['pi_num'] == $item->pi_rnum && $item->pi_status == 1){
                                        $poItemUpdateArr['pi_status'] = 2;
                                    }
                                    $dbConnect->table('poitems')
                                        ->where('pi_id',$item->pi_id)
                                        ->update($poItemUpdateArr);

                                    // 新增退货的采购订单信息
                                    $poItemCreateArr = [];
                                    foreach($item as $key => $ite){
                                        $poItemCreateArr[$key] = $ite;
                                    }

                                    unset($poItemCreateArr['pi_id']);
                                    $poItemCreateArr['pi_num'] = $refundInfo->refund_nums;
                                    $poItemCreateArr['pi_amount'] = bcmul($item->pi_price, $poItemCreateArr['pi_num'], 2);
                                    $poItemCreateArr['pi_rnum'] = $refundInfo->refund_nums;
                                    $poItemCreateArr['pi_status'] = $poItemCreateArr['pi_num'] == $poItemCreateArr['pi_rnum'] ? 2 : 1;

                                    $poItemsId = $dbConnect->table('poitems')->insertGetId($poItemCreateArr);
                                    //将soitems表中的新增记录中的pi_id修改为新值
                                    $newItem->update([
                                        'pi_id' => $poItemsId
                                    ]);
                                    if($refundInfo->cancel_porder == 1){
                                        // 取消采购订单
                                        $this->cancelPorderItems($dbConnect,$poItemCreateArr['po_id'],$poItemsId);
                                    }
                                }
                            }
                        }
                    }

                    //将dpdetail表里的新记录的si_id改为新值
                    /*$newDpdetail = $dbConnect->table('dpdetail')->where('dpd_id',$dpId)->first();
                    if(!$newDpdetail){
                        throw new \LogicException('数据处理失败！',500);
                    }
                    $updateDpArr = [];
                    $updateDpArr['si_id'] = $newItem->si_id;

                    $dbConnect->table('dpdetail')
                        ->where('dpd_id',$dpId)
                        ->update($updateDpArr);

                    */

                    //预存款和退款处理
                    $this->cancelAndTransPrePay($dbConnect,$newItem->toArray()['si_id'],$refundInfo);
                }
            }

            $dbConnect->commit();

        }catch(\Exception $e){
            $dbConnect->rollBack();
            throw $e;
        }

    }


    /**
     * 具体处理拆分dpdetail数据
     * @param $dbConnect
     * @param $recordsArr
     * @param $si_id
     * @param $createArr
     * @param $updateArr
     * @param $refundInfo
     * @param string $sort
     * @return array
     */
    protected function handleDpdetailInfo($dbConnect, &$num, $recordsArr, $si_id, $createArr, $updateArr, $refundInfo, $sort = 'arsort')
    {
        $cancelRecord = [];
        if(!empty($recordsArr) && count($recordsArr) > 0){
            foreach($recordsArr as $records){
                if(is_array(current($records))){
                    $sort($records);
                    foreach($records as $re){
                        if($num > 0){
                            // 退货数量 > 当前记录的数量 或者 退货数量 = 当前记录的数量
                            if(($num > $re['dpd_num']) || ($num == $re['dpd_num'])){
                                $num = $num - $re['dpd_num'];
                                $dpUpdateArr = [];
                                $dpUpdateArr['si_id'] = $si_id;
                                if($refundInfo->handle_fee > 0){
                                    $dpUpdateArr['dpd_amount'] = bcmul($createArr['si_discount'], $re['dpd_num'], 2);
                                }
                                $dbConnect->table('dpdetail')
                                    ->where('dpd_id',$re['dpd_id'])
                                    ->update($dpUpdateArr);

                                $cancelRecord[] = $re['dpd_id'];

                            } // 如果退货数量 < 当前记录的数量
                            elseif($num < $re['dpd_num']){

                                // 更新当前记录
                                $dpUpdateArr = [];
                                $dpUpdateArr['dpd_num'] = $re['dpd_num'] - $num;
                                if($refundInfo->handle_fee > 0){
                                    $dpUpdateArr['dpd_amount'] = bcmul($updateArr['si_discount'], $dpUpdateArr['dpd_num'], 2);
                                }
                                $dbConnect->table('dpdetail')
                                    ->where('dpd_id',$re['dpd_id'])
                                    ->update($dpUpdateArr);

                                $cancelRecord[] = $re['dpd_id'];

                                //添加新的取消记录
                                $dpCreateArr = $re;
                                unset($dpCreateArr['dpd_id']);
                                $dpCreateArr['dpd_num'] = $num;
                                if($refundInfo->handle_fee > 0){
                                    $dpCreateArr['dpd_amount'] = bcmul($createArr['si_discount'], $num, 2);
                                }
                                $dpCreateArr['si_id'] = $si_id;
                                $dpId = $dbConnect->table('dpdetail')->insertGetId($dpCreateArr);

                                $num = 0;
                                $cancelRecord[] = $dpId;
                            }

                        }
                    }
                }
            }
        }

        return $cancelRecord;
    }

    /**
     * 取消采购订单项
     * @param $db
     * @param $po_id
     * @param $pi_id
     */
    protected function cancelPorderItems($db,$po_id,$pi_id)
    {
        $pOrderItems = app(BhPordersRepository::class)->getPorderItemsFromPoId($po_id,$pi_id);

        $total = $db->table('poitems')
            ->where('po_id',$po_id)
            ->where('pi_status','<>',3)
            ->where('pi_id','<>',$pi_id)
            ->sum('pi_amount');

        //$row = $db->select("SELECT SUM(`pi_amount`) AS `total` FROM `poitems` WHERE `po_id` = ".$po_id." AND `pi_id` <> ".$pi_id." AND `pi_status` <> 3");
        //$row = current($row);
        //判断是否是最后一条非运费明细
        if($total > 0){
            $total += current($pOrderItems)['po_express'];
            $db->table('porders')->where('po_id', $po_id)->update(['po_total'=>$total]);
            $db->table('poitems')->where('pi_id', $pi_id)->update(['pi_status'=>'3']);
        }else{
            $db->table('porders')
                ->where('po_id', $po_id)
                ->update(
                    [
                        'po_total' => '0',
                        'po_review_status' => '2'
                    ]
                );
            $db->table('poitems')->where('pi_id', $pi_id)->update(['pi_status'=>'3']);
        }

        //更新采购订单的开票状态
        $pOrderItems = current($pOrderItems);
        if($pOrderItems['po_invoiced_amount'] >= $pOrderItems['po_total']){
            $db->table('porders')->where('po_id',$po_id)->update(['po_invoice_status' => 2]);
        }elseif($pOrderItems['po_invoiced_amount'] > 0){
            $db->table('porders')->where('po_id',$po_id)->update(['po_invoice_status' => 1]);
        }else{
            $db->table('porders')->where('po_id',$po_id)->update(['po_invoice_status' => 0]);
        }

        return true;
    }

    /**
     * 取消并转预存
     * @param $db
     * @param $si_id
     */
    protected function cancelAndTransPrePay($db,$si_id,$refundInfo)
    {
        //获取订单明细项
        $sOrderItem = app(BhSoitemsRepository::class)->getSorderItemFromItemId($si_id);
        $sOrderItem = current($sOrderItem->toArray());
        if($sOrderItem['si_if_cancel'] == 1){
            throw new \LogicException('已经取消过了，请不要重复操作！',500);
        }

        $sOrderDetail = app(BhSordersRepository::class)->getSorderDetailFromOrderId($sOrderItem['so_id']);
        if(!$sOrderDetail){
            throw new \LogicException('数据不存在!',500);
        }
        $sOrderDetail['express'] = 0;
        foreach($sOrderDetail['items'] as $k=>$v){
            if($v->p_id == 0){
                $sOrderDetail['express'] = $v->si_amount;
                unset($sOrderDetail['items'][$k]);
            }
        }

        /*if($sOrderDetail['so_pay_status'] == 0){
            throw new \LogicException('未完成付款的订单不能取消转预存款！',500);
        }*/
        if($sOrderDetail['so_pay_status'] == 1){
            throw new \LogicException('部分付款的订单不能取消明细，请联系管理员！',500);
        }

        if($sOrderItem['pi_id'] > 0){
            //判断对应的采购订单明细是否已经取消
            $pOrderItems = $db->table('poitems')
                ->join('porders','poitems.po_id','porders.po_id')
                ->where('poitems.pi_id',$sOrderItem['pi_id'])
                ->first();
            if(!$pOrderItems){
                throw new \LogicException('查询采购订单明细 ['.$sOrderItem['pi_id'].'] 数据不存在!',500);
            }
            if($pOrderItems->pi_status != 3 && $refundInfo->cancel_porder == 1){
                throw new \LogicException('请先取消采购订单：'.$pOrderItems->po_no.'中的对应采购订单明细！');
            }
        }

        if($sOrderItem['si_assigned_num'] > 0){
            $dp_status = array();
            $invoiced_amount = 0;
            $dpdetailInfos = $db->table('dpdetail')
                ->join('dispatch','dpdetail.dp_id','dispatch.dp_id')
                ->where('dpdetail.si_id',$sOrderItem['si_id'])
                ->get();
            if(!$dpdetailInfos){
               throw new \LogicException('dpdetail ['.$sOrderItem['si_id'].'] 配货单数据不存在!',500);
            }
            foreach($dpdetailInfos as $k=>$v){
                $dp_status[] = $v->dp_status;
                $invoiced_amount += $v->dpd_invoiced;
            }
            if($invoiced_amount != 0){
                throw new \LogicException('已经开过发票，不能取消！',500);
            }
            $dp_status = array_unique($dp_status);
            if(in_array('0',$dp_status) || in_array('1',$dp_status)){
                throw new \LogicException('请物流部配合处理至已发货状态，再做取消操作！',500);
            }
        }

        // 具体处理退货并转预存
        $si_cancel_time = 0;

        $siAmountTotal = $db->table('soitems')
            ->where('so_id',$sOrderItem['so_id'])
            ->where('si_id','<>',$si_id)
            ->where('si_if_cancel',0)
            ->sum('si_amount');
        $siAmountTotalPid = $db->table('soitems')
            ->where('so_id',$sOrderItem['so_id'])
            ->where('si_id','<>',$si_id)
            ->where('si_if_cancel',0)
            ->where('p_id','>',0)
            ->sum('si_amount');

        $soItem = app(BhSoitemsRepository::class)->find($si_id);
        if($soItem){
            if($soItem['si_assigned_num']){
                $dp = $db->table('dpdetail')->where('si_id',$si_id)->get();
                $dp = $dp->toArray();
                foreach($dp as $k=>$v){
                    if($v->dp_id == 0 && $v->stk_id > 0){
                        //清除已配货记录
                        $stock = $db->table('stock')->where('stk_id',$v['stk_id'])->get();
                        $stock = current($stock->toArray());
                        $stock_data = array();
                        $stock_data['stk_lock_num'] = $stock->stk_lock_num - $v->dpd_num;
                        if($stock_data['stk_lock_num'] < 0){
                            $stock_data['stk_lock_num'] = 0;
                        }
                        $db->table('stock')
                            ->where('stk_id',$v->stk_id)
                            ->update($stock_data);

                        $db->table('dpdetail')
                            ->delete($v->dpd_id);

                        $si_assigned_num = $soItem['si_assigned_num'] - $v->dpd_num;

                        app(BhSoitemsRepository::class)->update(['si_patch_status'=>'0','si_assigned_num'=>$si_assigned_num],$si_id);
                    }
                }
            }
        }
        $customer = $db->table('customers')
            ->where('cus_id',$sOrderDetail['cus_id'])
            ->first();
        //判断是否是最后一条非运费明细
        $note = $sOrderDetail['so_no'].'取消明细：'.$soItem['p_code'];
        if($siAmountTotalPid > 0){
            $refund_amount = $soItem['si_amount'];

            if($sOrderDetail['so_pay_status'] > 0){
                // 退款
                if($refundInfo->transfer_pre == 0){

                    $refund_data = [];
                    $refund_data['r_company'] = $customer->cus_company;
                    $refund_data['r_amount'] = $refund_amount;
                    $refund_data['r_ctime'] = time();
                    $refund_data['r_note'] = $note;
                    $refund_data['r_bank_account'] = '';
                    $refund_data['r_confirm_note'] = '';
                    $db->table('refund')->insertGetId($refund_data);

                }else{

                    $prepaylog_data = [];
                    $prepaylog_data['cus_id'] = $sOrderDetail['cus_id'];
                    $prepaylog_data['prepaylog_amount'] = $refund_amount;
                    $prepaylog_data['prepaylog_note'] = $note;
                    $prepaylog_data['prepaylog_date'] = time();
                    $prepaylog_data['prepaylog_time'] = time();
                    $prepaylog_data['prepaylog_operator'] = auth()->user()->id;

                    $prepayId = $db->table('prepaylog')->insertGetId($prepaylog_data);

                    $sum = $db->table('prepaylog')
                        ->where('cus_id',$sOrderDetail['cus_id'])
                        ->sum('prepaylog_amount');

                    $db->table('customers')
                        ->where('cus_id',$sOrderDetail['cus_id'])
                        ->update(['cus_prepay' => $sum]);

                    $si_cancel_time = time();
                }
            }

            app(BhSordersRepository::class)->update(['so_total'=>$siAmountTotal],$sOrderItem['so_id']);
            app(BhSoitemsRepository::class)->update(['si_if_cancel'=>'1','si_cancel_time'=>$si_cancel_time],$si_id);

        }else {

            if($sOrderDetail['so_pay_status'] > 0){
                //得到退款金额
                $sOrder = app(BhSordersRepository::class)->find($sOrderItem['so_id']);
                if (!$sOrder) {
                    throw new \LogicException('数据不存在！', 500);
                }
                $sOrder = $sOrder->toArray();

                $refund_amount = $sOrder['so_total'];

                if($refundInfo->transfer_pre == 0){

                    $refund_data = array();
                    $refund_data['r_company'] = $customer->cus_company;
                    $refund_data['r_amount'] = $refund_amount;
                    $refund_data['r_ctime'] = time();
                    $refund_data['r_note'] = $note;
                    $refund_data['r_bank_account'] = '';
                    $refund_data['r_confirm_note'] = '';
                    $db->table('refund')->insertGetId($refund_data);

                }else{

                    $prepaylog_data = [];
                    $prepaylog_data['cus_id'] = $sOrderDetail['cus_id'];
                    $prepaylog_data['prepaylog_amount'] = $refund_amount;
                    $prepaylog_data['prepaylog_note'] = $note;
                    $prepaylog_data['prepaylog_date'] = time();
                    $prepaylog_data['prepaylog_time'] = time();
                    $prepaylog_data['prepaylog_operator'] = auth()->user()->id;

                    $prepayId = $db->table('prepaylog')->insertGetId($prepaylog_data);

                    $sum = $db->table('prepaylog')
                        ->where('cus_id', $sOrderDetail['cus_id'])
                        ->sum('prepaylog_amount');

                    $db->table('customers')
                        ->where('cus_id', $sOrderDetail['cus_id'])
                        ->update(['cus_prepay' => $sum]);

                    $si_cancel_time = time();
                }
            }

            app(BhSordersRepository::class)->update(['so_review_status' => '3', 'so_total' => '0'], $sOrderItem['so_id']);
            app(BhSoitemsRepository::class)->update(['si_if_cancel' => '1', 'si_cancel_time' => $si_cancel_time], $sOrderItem['so_id']);
        }

        $db->table('logs')->insertGetId(
            ['l_obj'=>$sOrderItem['so_id'],'l_type'=>'sorders_mng','l_op'=>'cancelsi','l_op_name'=>'取消明细','l_note'=>$note,'l_timestamp'=>time(),'u_id'=>auth()->user()->id]
        );

        return true;
    }


    /**
     * 处理需要返回的数据
     * @param $pOrdersList
     * @return array|\Illuminate\Support\Collection
     */
    protected function procOrderResponseData($items)
    {
        $items = collect($items)->map(function($item){
            $item = $item->toArray();

            //开票状态
            switch($item['po_invoice_status']){
                case 0 : $item['po_invoice_status'] = '未开票';break;
                case 1 : $item['po_invoice_status'] = '部分开票';break;
                case 2 : $item['po_invoice_status'] = '全额开票';break;
            }

            //是否直发
            switch($item['po_if_direct']){
                case 0 : $item['po_if_direct_label'] = '否'; break;
                case 1 : $item['po_if_direct_label'] = '是'; break;
            }

            //订单类型
            switch($item['po_type']){
                case 0 : $item['po_type'] = '备货'; break;
                case 1 : $item['po_type'] = '订单'; break;
            }

            //审核状态
            switch($item['po_review_status']){
                case 0 : $item['po_review_status'] = '未付款'; break;
                case 1 : $item['po_review_status'] = '部分付款'; break;
                case 2 : $item['po_review_status'] = '全额付款'; break;
            }

            //付款类型
            switch($item['po_pay_type']){
                case 0 : $item['po_pay_type'] = '公账'; break;
                case 1 : $item['po_pay_type'] = '私账'; break;
                case 2 : $item['po_pay_type'] = '预付款'; break;
            }

            return $item;
        });

        return $items;

    }

}
